module xillyblaze
  (
   output ddr_memory_we_n,
   output ddr_memory_ras_n,
   output ddr_memory_odt,
   inout [0:0] ddr_memory_dqs_n,
   inout [0:0] ddr_memory_dqs,
   inout [7:0] ddr_memory_dq,
   output [0:0] ddr_memory_dm,
   output ddr_memory_ddr3_rst,
   output ddr_memory_cs_n,
   output ddr_memory_clk_n,
   output ddr_memory_clk,
   output ddr_memory_cke,
   output ddr_memory_cas_n,
   output [2:0] ddr_memory_ba,
   output [12:0] ddr_memory_addr,
   output SysACE_WEN,
   output SysACE_OEN,
   input SysACE_MPIRQ,
   inout [7:0] SysACE_MPD,
   output [6:0] SysACE_MPA,
   input SysACE_CLK,
   output SysACE_CEN,
   output RS232_Uart_1_sout,
   input RS232_Uart_1_sin,
   input RESET,
   input [0:4] Push_Buttons_5Bits_TRI_I,
   output Linear_Flash_we_n,
   output Linear_Flash_oe_n,
   inout [0:15] Linear_Flash_data,
   output Linear_Flash_ce_n,
   output [0:23] Linear_Flash_address,
   output [0:4] LEDs_Positions_TRI_O,
   output [0:7] LEDs_8Bits_TRI_O,
   inout IIC_SFP_SDA,
   inout IIC_SFP_SCL,
   inout IIC_FMC_SDA,
   inout IIC_FMC_SCL,
   inout IIC_EEPROM_SDA,
   inout IIC_EEPROM_SCL,
   inout IIC_DVI_SDA,
   inout IIC_DVI_SCL,
   output Ethernet_Lite_TX_EN,
   input Ethernet_Lite_TX_CLK,
   output [3:0] Ethernet_Lite_TXD,
   input Ethernet_Lite_RX_ER,
   input Ethernet_Lite_RX_DV,
   input Ethernet_Lite_RX_CLK,
   input [3:0] Ethernet_Lite_RXD,
   output Ethernet_Lite_PHY_RST_N,
   inout Ethernet_Lite_MDIO,
   output Ethernet_Lite_MDC,
   input Ethernet_Lite_CRS,
   input Ethernet_Lite_COL,
   input [0:7] DIP_Switches_8Bits_TRI_I,
   input CLK_P,
   input CLK_N
   );

   // Clock and quiesce
   wire    bus_clk;
   wire    quiesce;

   // Memory array
   reg [7:0] demoarray[0:31];

   // Wires related to /dev/xillybus_mem_8
   wire      user_r_mem_8_rden;
   wire      user_r_mem_8_empty;
   reg [7:0] user_r_mem_8_data;
   wire      user_r_mem_8_eof;
   wire      user_r_mem_8_open;
   wire      user_w_mem_8_wren;
   wire      user_w_mem_8_full;
   wire [7:0] user_w_mem_8_data;
   wire       user_w_mem_8_open;
   wire [4:0] user_mem_8_addr;
   wire       user_mem_8_addr_update;

   // Wires related to /dev/xillybus_read_32
   wire       user_r_read_32_rden;
   wire       user_r_read_32_empty;
   wire [31:0] user_r_read_32_data;
   wire        user_r_read_32_eof;
   wire        user_r_read_32_open;

   // Wires related to /dev/xillybus_read_8
   wire        user_r_read_8_rden;
   wire        user_r_read_8_empty;
   wire [7:0]  user_r_read_8_data;
   wire        user_r_read_8_eof;
   wire        user_r_read_8_open;

   // Wires related to /dev/xillybus_write_32
   wire        user_w_write_32_wren;
   wire        user_w_write_32_full;
   wire [31:0] user_w_write_32_data;
   wire        user_w_write_32_open;

   // Wires related to /dev/xillybus_write_8
   wire        user_w_write_8_wren;
   wire        user_w_write_8_full;
   wire [7:0]  user_w_write_8_data;
   wire        user_w_write_8_open;

   xillybus xillybus_ins (
			  .bus_clk(bus_clk),
			  // Ports related to /dev/xillybus_mem_8
			  // FPGA to CPU signals:
			  .user_r_mem_8_rden(user_r_mem_8_rden),
			  .user_r_mem_8_empty(user_r_mem_8_empty),
			  .user_r_mem_8_data(user_r_mem_8_data),
			  .user_r_mem_8_eof(user_r_mem_8_eof),
			  .user_r_mem_8_open(user_r_mem_8_open),

			  // CPU to FPGA signals:
			  .user_w_mem_8_wren(user_w_mem_8_wren),
			  .user_w_mem_8_full(user_w_mem_8_full),
			  .user_w_mem_8_data(user_w_mem_8_data),
			  .user_w_mem_8_open(user_w_mem_8_open),

			  // Address signals:
			  .user_mem_8_addr(user_mem_8_addr),
			  .user_mem_8_addr_update(user_mem_8_addr_update),


			  // Ports related to /dev/xillybus_read_32
			  // FPGA to CPU signals:
			  .user_r_read_32_rden(user_r_read_32_rden),
			  .user_r_read_32_empty(user_r_read_32_empty),
			  .user_r_read_32_data(user_r_read_32_data),
			  .user_r_read_32_eof(user_r_read_32_eof),
			  .user_r_read_32_open(user_r_read_32_open),


			  // Ports related to /dev/xillybus_read_8
			  // FPGA to CPU signals:
			  .user_r_read_8_rden(user_r_read_8_rden),
			  .user_r_read_8_empty(user_r_read_8_empty),
			  .user_r_read_8_data(user_r_read_8_data),
			  .user_r_read_8_eof(user_r_read_8_eof),
			  .user_r_read_8_open(user_r_read_8_open),


			  // Ports related to /dev/xillybus_write_32
			  // CPU to FPGA signals:
			  .user_w_write_32_wren(user_w_write_32_wren),
			  .user_w_write_32_full(user_w_write_32_full),
			  .user_w_write_32_data(user_w_write_32_data),
			  .user_w_write_32_open(user_w_write_32_open),


			  // Ports related to /dev/xillybus_write_8
			  // CPU to FPGA signals:
			  .user_w_write_8_wren(user_w_write_8_wren),
			  .user_w_write_8_full(user_w_write_8_full),
			  .user_w_write_8_data(user_w_write_8_data),
			  .user_w_write_8_open(user_w_write_8_open),


			  // General signals

			  .ddr_memory_we_n ( ddr_memory_we_n ),
			  .ddr_memory_ras_n ( ddr_memory_ras_n ),
			  .ddr_memory_odt ( ddr_memory_odt ),
			  .ddr_memory_dqs_n ( ddr_memory_dqs_n[0:0] ),
			  .ddr_memory_dqs ( ddr_memory_dqs[0:0] ),
			  .ddr_memory_dq ( ddr_memory_dq ),
			  .ddr_memory_dm ( ddr_memory_dm[0:0] ),
			  .ddr_memory_ddr3_rst ( ddr_memory_ddr3_rst ),
			  .ddr_memory_cs_n ( ddr_memory_cs_n ),
			  .ddr_memory_clk_n ( ddr_memory_clk_n ),
			  .ddr_memory_clk ( ddr_memory_clk ),
			  .ddr_memory_cke ( ddr_memory_cke ),
			  .ddr_memory_cas_n ( ddr_memory_cas_n ),
			  .ddr_memory_ba ( ddr_memory_ba ),
			  .ddr_memory_addr ( ddr_memory_addr ),
			  .SysACE_WEN ( SysACE_WEN ),
			  .SysACE_OEN ( SysACE_OEN ),
			  .SysACE_MPIRQ ( SysACE_MPIRQ ),
			  .SysACE_MPD ( SysACE_MPD ),
			  .SysACE_MPA ( SysACE_MPA ),
			  .SysACE_CLK ( SysACE_CLK ),
			  .SysACE_CEN ( SysACE_CEN ),
			  .RS232_Uart_1_sout ( RS232_Uart_1_sout ),
			  .RS232_Uart_1_sin ( RS232_Uart_1_sin ),
			  .RESET ( RESET ),
			  .Push_Buttons_5Bits_TRI_I ( Push_Buttons_5Bits_TRI_I ),
			  .Linear_Flash_we_n ( Linear_Flash_we_n ),
			  .Linear_Flash_oe_n ( Linear_Flash_oe_n ),
			  .Linear_Flash_data ( Linear_Flash_data ),
			  .Linear_Flash_ce_n ( Linear_Flash_ce_n ),
			  .Linear_Flash_address ( Linear_Flash_address ),
			  .LEDs_Positions_TRI_O ( LEDs_Positions_TRI_O ),
			  .LEDs_8Bits_TRI_O ( LEDs_8Bits_TRI_O ),
			  .IIC_SFP_SDA ( IIC_SFP_SDA ),
			  .IIC_SFP_SCL ( IIC_SFP_SCL ),
			  .IIC_FMC_SDA ( IIC_FMC_SDA ),
			  .IIC_FMC_SCL ( IIC_FMC_SCL ),
			  .IIC_EEPROM_SDA ( IIC_EEPROM_SDA ),
			  .IIC_EEPROM_SCL ( IIC_EEPROM_SCL ),
			  .IIC_DVI_SDA ( IIC_DVI_SDA ),
			  .IIC_DVI_SCL ( IIC_DVI_SCL ),
			  .Ethernet_Lite_TX_EN ( Ethernet_Lite_TX_EN ),
			  .Ethernet_Lite_TX_CLK ( Ethernet_Lite_TX_CLK ),
			  .Ethernet_Lite_TXD ( Ethernet_Lite_TXD ),
			  .Ethernet_Lite_RX_ER ( Ethernet_Lite_RX_ER ),
			  .Ethernet_Lite_RX_DV ( Ethernet_Lite_RX_DV ),
			  .Ethernet_Lite_RX_CLK ( Ethernet_Lite_RX_CLK ),
			  .Ethernet_Lite_RXD ( Ethernet_Lite_RXD ),
			  .Ethernet_Lite_PHY_RST_N ( Ethernet_Lite_PHY_RST_N ),
			  .Ethernet_Lite_MDIO ( Ethernet_Lite_MDIO ),
			  .Ethernet_Lite_MDC ( Ethernet_Lite_MDC ),
			  .Ethernet_Lite_CRS ( Ethernet_Lite_CRS ),
			  .Ethernet_Lite_COL ( Ethernet_Lite_COL ),
			  .DIP_Switches_8Bits_TRI_I ( DIP_Switches_8Bits_TRI_I ),
			  .CLK_P ( CLK_P ),
			  .CLK_N ( CLK_N ),

			  .quiesce(quiesce)
			  );

   
   // A simple inferred RAM
   always @(posedge bus_clk)
     begin
	if (user_w_mem_8_wren)
	  demoarray[user_mem_8_addr] <= user_w_mem_8_data;
	
	if (user_r_mem_8_rden)
	  user_r_mem_8_data <= demoarray[user_mem_8_addr];	  
     end

   assign  user_r_mem_8_empty = 0;
   assign  user_r_mem_8_eof = 0;
   assign  user_w_mem_8_full = 0;

   // 32-bit loopback
   fifo_32x512 fifo_32
     (
      .clk(bus_clk),
      .srst(!user_w_write_32_open && !user_r_read_32_open),
      .din(user_w_write_32_data),
      .wr_en(user_w_write_32_wren),
      .rd_en(user_r_read_32_rden),
      .dout(user_r_read_32_data),
      .full(user_w_write_32_full),
      .empty(user_r_read_32_empty)
      );

   assign  user_r_read_32_eof = 0;
   
   // 8-bit loopback
   fifo_8x2048 fifo_8
     (
      .clk(bus_clk),
      .srst(!user_w_write_8_open && !user_r_read_8_open),
      .din(user_w_write_8_data),
      .wr_en(user_w_write_8_wren),
      .rd_en(user_r_read_8_rden),
      .dout(user_r_read_8_data),
      .full(user_w_write_8_full),
      .empty(user_r_read_8_empty)
      );

   assign  user_r_read_8_eof = 0;
   
endmodule
